@@ -1,5 +1,6 @@
 import glob
 import os
+import platform
 import subprocess
 import sys
 from distutils.version import StrictVersion
@@ -9,11 +10,12 @@ import coverage
 import py
 import pytest
 import virtualenv
+import xdist
+from fields import Namespace
 from process_tests import TestProcess as _TestProcess
 from process_tests import dump_on_error
 from process_tests import wait_for_strings
 from six import exec_
-from fields import Namespace
 
 try:
     from StringIO import StringIO
@@ -22,16 +24,16 @@ except ImportError:
 
 import pytest_cov.plugin
 
-coverage, StrictVersion  # required for skipif mark on test_cov_min_from_coveragerc
+coverage, platform, StrictVersion  # required for skipif mark on test_cov_min_from_coveragerc
 
 SCRIPT = '''
 import sys, helper
 
 def pytest_generate_tests(metafunc):
-    for i in range(10):
-        metafunc.addcall()
+    for i in [10]:
+        metafunc.parametrize('p', range(i))
 
-def test_foo():
+def test_foo(p):
     x = True
     helper.do_stuff()  # get some coverage in some other completely different location
     if sys.version_info[0] > 5:
@@ -70,8 +72,8 @@ import subprocess
 import sys
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     out, err = subprocess.Popen(
@@ -90,8 +92,8 @@ import sys
 import os
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     os.mkdir("foobar")
@@ -114,8 +116,8 @@ import sys
 import os
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        if metafunc.function is test_foo: metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     os.mkdir("foobar")
@@ -135,7 +137,7 @@ SCRIPT_FUNCARG = '''
 import coverage
 
 def test_foo(cov):
-    assert isinstance(cov, coverage.coverage)
+    assert isinstance(cov, coverage.Coverage)
 '''
 
 SCRIPT_FUNCARG_NOT_ACTIVE = '''
@@ -143,18 +145,32 @@ def test_foo(cov):
     assert cov is None
 '''
 
-SCRIPT_FAIL = '''
-def test_fail():
-    assert False
-
-'''
-
 CHILD_SCRIPT_RESULT = '[56] * 100%'
 PARENT_SCRIPT_RESULT = '9 * 100%'
 DEST_DIR = 'cov_dest'
 REPORT_NAME = 'cov.xml'
 
-xdist = pytest.mark.parametrize('opts', ['', '-n 1'], ids=['nodist', 'xdist'])
+xdist_params = pytest.mark.parametrize('opts', [
+    '',
+    pytest.param('-n 1', marks=pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"'))
+], ids=['nodist', 'xdist'])
+
+
+@pytest.fixture(scope='session', autouse=True)
+def adjust_sys_path():
+    """Adjust PYTHONPATH during tests to make "helper" importable in SCRIPT."""
+    orig_path = os.environ.get('PYTHONPATH', None)
+    new_path = os.path.dirname(__file__)
+    if orig_path is not None:
+        new_path = os.pathsep.join([new_path, orig_path])
+    os.environ['PYTHONPATH'] = new_path
+
+    yield
+
+    if orig_path is None:
+        del os.environ['PYTHONPATH']
+    else:
+        os.environ['PYTHONPATH'] = orig_path
 
 
 @pytest.fixture(params=[
@@ -396,7 +412,7 @@ def test_central_coveragerc(testdir, prop):
     assert result.ret == 0
 
 
-@xdist
+@xdist_params
 def test_central_with_path_aliasing(testdir, monkeypatch, opts, prop):
     mod1 = testdir.mkdir('src').join('mod.py')
     mod1.write(SCRIPT)
@@ -455,7 +471,8 @@ source =
 parallel = true
 """)
 
-    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([os.environ.get('PYTHONPATH',''), 'aliased']))
+    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([
+        os.environ.get('PYTHONPATH', ''), 'aliased']))
     result = testdir.runpytest('-v',
                                '--cov',
                                '--cov-report=term-missing',
@@ -499,7 +516,11 @@ show_missing = true
 
 
 def test_no_cov_on_fail(testdir):
-    script = testdir.makepyfile(SCRIPT_FAIL)
+    script = testdir.makepyfile('''
+def test_fail():
+    assert False
+
+''')
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -527,7 +548,11 @@ def test_no_cov(testdir):
 
 
 def test_cov_and_failure_report_on_fail(testdir):
-    script = testdir.makepyfile(SCRIPT + SCRIPT_FAIL)
+    script = testdir.makepyfile(SCRIPT + '''
+def test_fail(p):
+    assert False
+
+''')
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -542,6 +567,7 @@ def test_cov_and_failure_report_on_fail(testdir):
     ])
 
 
+@pytest.mark.skipif('sys.platform == "win32" or platform.python_implementation() == "PyPy"')
 def test_dist_combine_racecondition(testdir):
     script = testdir.makepyfile("""
 import pytest
@@ -570,6 +596,7 @@ def test_foo(foo):
     assert result.ret == 0
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_collocated(testdir, prop):
     script = testdir.makepyfile(prop.code)
     testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
@@ -589,6 +616,7 @@ def test_dist_collocated(testdir, prop):
     assert result.ret == 0
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_not_collocated(testdir, prop):
     script = testdir.makepyfile(prop.code)
     dir1 = testdir.mkdir('dir1')
@@ -621,6 +649,7 @@ source =
     assert result.ret == 0
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_not_collocated_coveragerc_source(testdir, prop):
     script = testdir.makepyfile(prop.code)
     dir1 = testdir.mkdir('dir1')
@@ -744,6 +773,7 @@ parallel = true
     assert result.ret == 0
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_subprocess_collocated(testdir):
     scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
                                  child_script=SCRIPT_CHILD)
@@ -765,6 +795,7 @@ def test_dist_subprocess_collocated(testdir):
     assert result.ret == 0
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_subprocess_not_collocated(testdir, tmpdir):
     scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
                                  child_script=SCRIPT_CHILD)
@@ -826,11 +857,15 @@ def test_invalid_coverage_source(testdir):
 
 
 @pytest.mark.skipif("'dev' in pytest.__version__")
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_dist_missing_data(testdir):
     venv_path = os.path.join(str(testdir.tmpdir), 'venv')
     virtualenv.create_environment(venv_path)
     if sys.platform == 'win32':
-        exe = os.path.join(venv_path, 'Scripts', 'python.exe')
+        if platform.python_implementation() == "PyPy":
+            exe = os.path.join(venv_path, 'bin', 'python.exe')
+        else:
+            exe = os.path.join(venv_path, 'Scripts', 'python.exe')
     else:
         exe = os.path.join(venv_path, 'bin', 'python')
     subprocess.check_call([
@@ -838,7 +873,9 @@ def test_dist_missing_data(testdir):
         '-mpip',
         'install',
         'py==%s' % py.__version__,
-        'pytest==%s' % pytest.__version__
+        'pytest==%s' % pytest.__version__,
+        'pytest_xdist==%s' % xdist.__version__
+
     ])
     script = testdir.makepyfile(SCRIPT)
 
@@ -884,8 +921,131 @@ def test_funcarg_not_active(testdir):
     assert result.ret == 0
 
 
-def test_multiprocessing_subprocess(testdir):
-    py.test.importorskip('multiprocessing.util')
+@pytest.mark.skipif("sys.version_info[0] < 3", reason="no context manager api on Python 2")
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+@pytest.mark.skipif('platform.python_implementation() == "PyPy"', reason="often deadlocks on PyPy")
+def test_multiprocessing_pool(testdir):
+    pytest.importorskip('multiprocessing.util')
+
+    script = testdir.makepyfile('''
+import multiprocessing
+
+def target_fn(a):
+    %sse:  # pragma: nocover
+        return None
+
+def test_run_target():
+    from pytest_cov.embed import cleanup_on_sigterm
+    cleanup_on_sigterm()
+
+    for i in range(33):
+        with multiprocessing.Pool(3) as p:
+            p.map(target_fn, [i * 3 + j for j in range(3)])
+        p.join()
+''' % ''.join('''if a == %r:
+        return a
+    el''' % i for i in range(99)))
+
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    assert "Doesn't seem to be a coverage.py data file" not in result.stdout.str()
+    assert "Doesn't seem to be a coverage.py data file" not in result.stderr.str()
+    assert not testdir.tmpdir.listdir(".coverage.*")
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_pool* 100%*',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+@pytest.mark.skipif('platform.python_implementation() == "PyPy"', reason="often deadlocks on PyPy")
+def test_multiprocessing_pool_terminate(testdir):
+    pytest.importorskip('multiprocessing.util')
+
+    script = testdir.makepyfile('''
+import multiprocessing
+
+def target_fn(a):
+    %sse:  # pragma: nocover
+        return None
+
+def test_run_target():
+    from pytest_cov.embed import cleanup_on_sigterm
+    cleanup_on_sigterm()
+
+    for i in range(33):
+        p = multiprocessing.Pool(3)
+        try:
+            p.map(target_fn, [i * 3 + j for j in range(3)])
+        finally:
+            p.terminate()
+            p.join()
+''' % ''.join('''if a == %r:
+        return a
+    el''' % i for i in range(99)))
+
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    assert "Doesn't seem to be a coverage.py data file" not in result.stdout.str()
+    assert "Doesn't seem to be a coverage.py data file" not in result.stderr.str()
+    assert not testdir.tmpdir.listdir(".coverage.*")
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_pool* 100%*',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+def test_multiprocessing_pool_close(testdir):
+    pytest.importorskip('multiprocessing.util')
+
+    script = testdir.makepyfile('''
+import multiprocessing
+
+def target_fn(a):
+    %sse:  # pragma: nocover
+        return None
+
+def test_run_target():
+    for i in range(33):
+        p = multiprocessing.Pool(3)
+        try:
+            p.map(target_fn, [i * 3 + j for j in range(3)])
+        finally:
+            p.close()
+            p.join()
+''' % ''.join('''if a == %r:
+        return a
+    el''' % i for i in range(99)))
+
+    result = testdir.runpytest('-v',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert "Doesn't seem to be a coverage.py data file" not in result.stdout.str()
+    assert "Doesn't seem to be a coverage.py data file" not in result.stderr.str()
+    assert not testdir.tmpdir.listdir(".coverage.*")
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_pool* 100%*',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+def test_multiprocessing_process(testdir):
+    pytest.importorskip('multiprocessing.util')
 
     script = testdir.makepyfile('''
 import multiprocessing
@@ -907,14 +1067,15 @@ def test_run_target():
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'test_multiprocessing_subprocess* 8 * 100%*',
+        'test_multiprocessing_process* 8 * 100%*',
         '*1 passed*'
     ])
     assert result.ret == 0
 
 
-def test_multiprocessing_subprocess_no_source(testdir):
-    py.test.importorskip('multiprocessing.util')
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+def test_multiprocessing_process_no_source(testdir):
+    pytest.importorskip('multiprocessing.util')
 
     script = testdir.makepyfile('''
 import multiprocessing
@@ -936,16 +1097,15 @@ def test_run_target():
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'test_multiprocessing_subprocess* 8 * 100%*',
+        'test_multiprocessing_process* 8 * 100%*',
         '*1 passed*'
     ])
     assert result.ret == 0
 
 
-@pytest.mark.skipif('sys.platform == "win32"',
-                    reason="multiprocessing don't support clean process temination on Windows")
-def test_multiprocessing_subprocess_with_terminate(testdir):
-    py.test.importorskip('multiprocessing.util')
+@pytest.mark.skipif('sys.platform == "win32"', reason="multiprocessing support is broken on Windows")
+def test_multiprocessing_process_with_terminate(testdir):
+    pytest.importorskip('multiprocessing.util')
 
     script = testdir.makepyfile('''
 import multiprocessing
@@ -976,11 +1136,225 @@ def test_run_target():
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'test_multiprocessing_subprocess* 16 * 100%*',
+        'test_multiprocessing_process* 16 * 100%*',
         '*1 passed*'
     ])
     assert result.ret == 0
 
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="SIGTERM isn't really supported on Windows")
+def test_cleanup_on_sigterm(testdir):
+    script = testdir.makepyfile('''
+import os, signal, subprocess, sys, time
+
+def cleanup(num, frame):
+    print("num == signal.SIGTERM => %s" % (num == signal.SIGTERM))
+    raise Exception()
+
+def test_run():
+    proc = subprocess.Popen([sys.executable, __file__], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    time.sleep(1)
+    proc.terminate()
+    stdout, stderr = proc.communicate()
+    assert not stderr
+    assert stdout == b"""num == signal.SIGTERM => True
+captured Exception()
+"""
+    assert proc.returncode == 0
+
+if __name__ == "__main__":
+    signal.signal(signal.SIGTERM, cleanup)
+
+    from pytest_cov.embed import cleanup_on_sigterm
+    cleanup_on_sigterm()
+
+    try:
+        time.sleep(10)
+    except BaseException as exc:
+        print("captured %r" % exc)
+''')
+
+    result = testdir.runpytest('-vv',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_cleanup_on_sigterm* 26-27',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform != "win32"')
+@pytest.mark.parametrize('setup', [
+    ('signal.signal(signal.SIGBREAK, signal.SIG_DFL); cleanup_on_signal(signal.SIGBREAK)', '87%   21-22'),
+    ('cleanup_on_signal(signal.SIGBREAK)', '87%   21-22'),
+    ('cleanup()', '73%   19-22'),
+])
+def test_cleanup_on_sigterm_sig_break(testdir, setup):
+    # worth a read: https://stefan.sofa-rockers.org/2013/08/15/handling-sub-process-hierarchies-python-linux-os-x/
+    script = testdir.makepyfile('''
+import os, signal, subprocess, sys, time
+
+def test_run():
+    proc = subprocess.Popen(
+        [sys.executable, __file__],
+        stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+        creationflags=subprocess.CREATE_NEW_PROCESS_GROUP, shell=True
+    )
+    time.sleep(1)
+    proc.send_signal(signal.CTRL_BREAK_EVENT)
+    stdout, stderr = proc.communicate()
+    assert not stderr
+    assert stdout in [b"^C", b"", b"captured IOError(4, 'Interrupted function call')\\n"]
+
+if __name__ == "__main__":
+    from pytest_cov.embed import cleanup_on_signal, cleanup
+    ''' + setup[0] + '''
+
+    try:
+        time.sleep(10)
+    except BaseException as exc:
+        print("captured %r" % exc)
+''')
+
+    result = testdir.runpytest('-vv',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_cleanup_on_sigterm* %s' % setup[1],
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="SIGTERM isn't really supported on Windows")
+@pytest.mark.parametrize('setup', [
+    ('signal.signal(signal.SIGTERM, signal.SIG_DFL); cleanup_on_sigterm()', '88%   18-19'),
+    ('cleanup_on_sigterm()', '88%   18-19'),
+    ('cleanup()', '75%   16-19'),
+])
+def test_cleanup_on_sigterm_sig_dfl(testdir, setup):
+    script = testdir.makepyfile('''
+import os, signal, subprocess, sys, time
+
+def test_run():
+    proc = subprocess.Popen([sys.executable, __file__], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    time.sleep(1)
+    proc.terminate()
+    stdout, stderr = proc.communicate()
+    assert not stderr
+    assert stdout == b""
+    assert proc.returncode in [128 + signal.SIGTERM, -signal.SIGTERM]
+
+if __name__ == "__main__":
+    from pytest_cov.embed import cleanup_on_sigterm, cleanup
+    ''' + setup[0] + '''
+
+    try:
+        time.sleep(10)
+    except BaseException as exc:
+        print("captured %r" % exc)
+''')
+
+    result = testdir.runpytest('-vv',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_cleanup_on_sigterm* %s' % setup[1],
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="SIGINT is subtly broken on Windows")
+def test_cleanup_on_sigterm_sig_dfl_sigint(testdir):
+    script = testdir.makepyfile('''
+import os, signal, subprocess, sys, time
+
+def test_run():
+    proc = subprocess.Popen([sys.executable, __file__], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    time.sleep(1)
+    proc.send_signal(signal.SIGINT)
+    stdout, stderr = proc.communicate()
+    assert not stderr
+    assert stdout == b"""captured KeyboardInterrupt()
+"""
+    assert proc.returncode == 0
+
+if __name__ == "__main__":
+    from pytest_cov.embed import cleanup_on_signal
+    cleanup_on_signal(signal.SIGINT)
+
+    try:
+        time.sleep(10)
+    except BaseException as exc:
+        print("captured %r" % exc)
+''')
+
+    result = testdir.runpytest('-vv',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_cleanup_on_sigterm* 88%   19-20',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"', reason="fork not available on Windows")
+def test_cleanup_on_sigterm_sig_ign(testdir):
+    script = testdir.makepyfile('''
+import os, signal, subprocess, sys, time
+
+def test_run():
+    proc = subprocess.Popen([sys.executable, __file__], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    time.sleep(1)
+    proc.send_signal(signal.SIGINT)
+    time.sleep(1)
+    proc.terminate()
+    stdout, stderr = proc.communicate()
+    assert not stderr
+    assert stdout == b""
+    # it appears signal handling is buggy on python 2?
+    if sys.version_info == 3: assert proc.returncode in [128 + signal.SIGTERM, -signal.SIGTERM]
+
+if __name__ == "__main__":
+    signal.signal(signal.SIGINT, signal.SIG_IGN)
+
+    from pytest_cov.embed import cleanup_on_signal
+    cleanup_on_signal(signal.SIGINT)
+
+    try:
+        time.sleep(10)
+    except BaseException as exc:
+        print("captured %r" % exc)
+    ''')
+
+    result = testdir.runpytest('-vv',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_cleanup_on_sigterm* 89%   23-24',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
 MODULE = '''
 def func():
     return 1
@@ -1017,12 +1391,14 @@ def test_cover_conftest(testdir):
     result.stdout.fnmatch_lines([CONF_RESULT])
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_cover_looponfail(testdir, monkeypatch):
     testdir.makepyfile(mod=MODULE)
     testdir.makeconftest(CONFTEST)
     script = testdir.makepyfile(BASIC_TEST)
 
-    monkeypatch.setattr(testdir, 'run', lambda *args: _TestProcess(*map(str, args)))
+    monkeypatch.setattr(testdir, 'run',
+                        lambda *args, **kwargs: _TestProcess(*map(str, args)))
     with testdir.runpytest('-v',
                            '--cov=%s' % script.dirpath(),
                            '--looponfail',
@@ -1035,6 +1411,7 @@ def test_cover_looponfail(testdir, monkeypatch):
             )
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_cover_conftest_dist(testdir):
     testdir.makepyfile(mod=MODULE)
     testdir.makeconftest(CONFTEST)
@@ -1061,7 +1438,7 @@ import sys
 @pytest.mark.no_cover
 def test_basic():
     mod.func()
-    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])
 ''')
     result = testdir.runpytest('-v', '-ra', '--strict',
                                '--cov=%s' % script.dirpath(),
@@ -1080,7 +1457,7 @@ import sys
 
 def test_basic(no_cover):
     mod.func()
-    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])
 ''')
     result = testdir.runpytest('-v', '-ra', '--strict',
                                '--cov=%s' % script.dirpath(),
@@ -1124,6 +1501,7 @@ def test_coveragerc(testdir):
     result.stdout.fnmatch_lines(['test_coveragerc* %s' % EXCLUDED_RESULT])
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_coveragerc_dist(testdir):
     testdir.makefile('', coveragerc=COVERAGERC)
     script = testdir.makepyfile(EXCLUDED_TEST)
@@ -1312,6 +1690,7 @@ data_file = %s
     assert glob.glob(str(testdir.tmpdir.join('some/special/place/coverage-data*')))
 
 
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
 def test_external_data_file_xdist(testdir):
     script = testdir.makepyfile(SCRIPT)
     testdir.tmpdir.join('.coveragerc').write("""
@@ -1340,8 +1719,8 @@ def test_external_data_file_negative(testdir):
     assert glob.glob(str(testdir.tmpdir.join('.coverage*')))
 
 
-@xdist
-def xtest_append_coverage(testdir, opts, prop):
+@xdist_params
+def test_append_coverage(testdir, opts, prop):
     script = testdir.makepyfile(test_1=prop.code)
     testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
     result = testdir.runpytest('-v',
@@ -1363,10 +1742,10 @@ def xtest_append_coverage(testdir, opts, prop):
     ])
 
 
-@xdist
-def xtest_do_not_append_coverage(testdir, opts, prop):
+@xdist_params
+def test_do_not_append_coverage(testdir, opts, prop):
     script = testdir.makepyfile(test_1=prop.code)
-    testdir.tmpdir.join('.coveragerc').write("")
+    testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
                                script,
@@ -1433,3 +1812,14 @@ def test_double_cov2(testdir):
         '*1 passed*'
     ])
     assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32" and platform.python_implementation() == "PyPy"')
+def test_cov_and_no_cov(testdir):
+    script = testdir.makepyfile(SCRIPT_SIMPLE)
+    result = testdir.runpytest('-v',
+                               '--cov', '--no-cov',
+                               '-n', '1',
+                               script)
+
+    assert result.ret == 0
